"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleCore = void 0;
const core_types_1 = require("./core.types");
class RuleCore {
    constructor() {
        this.registerMany = (rules) => {
            rules.forEach((value, key) => this.registerOne(key, value));
        };
        this.registerOne = (ruleIdentifier, ruleHandler) => {
            this.rules.set(ruleIdentifier.name, ruleHandler);
            if (ruleIdentifier.shortName)
                this.rules.set(ruleIdentifier.shortName, ruleHandler);
        };
        this.execute = (jsonRules, data) => {
            const { rule, inputs, output } = this.getRuleParams(jsonRules);
            const ruleHandler = this.rules.get(rule);
            if (!ruleHandler)
                throw Error(`The "${rule}" is not exist`);
            const result = ruleHandler(inputs.map(input => this.getInputValue(input, data)), data);
            if (output)
                this.setOutputValue(output, result);
            return result;
        };
        this.executeAsync = (jsonRules, data) => __awaiter(this, void 0, void 0, function* () {
            const { rule, inputs, output } = this.getRuleParams(jsonRules);
            const ruleHandler = this.rules.get(rule);
            if (!ruleHandler)
                throw Error(`The "${rule}" is not exist`);
            try {
                const resolvedInputs = [];
                for (const input of inputs) {
                    const result = yield this.getInputValueAsync(input, data);
                    resolvedInputs.push(result);
                }
                const result = yield ruleHandler(resolvedInputs, data);
                if (output)
                    this.setOutputValue(output, result);
                return result;
            }
            catch (error) {
                throw Error(`Failed to Run "${rule}" cause of ${error}`);
            }
        });
        this.isRule = (data) => {
            return (typeof data === 'object' &&
                (data[core_types_1.RuleParams.Rule] || data[core_types_1.ShortRuleParams.Rule]) &&
                (data[core_types_1.RuleParams.Input] || data[core_types_1.ShortRuleParams.Input]));
        };
        this.getInputValue = (input, data) => {
            return this.isRule(input) ? this.execute(input, data) : input;
        };
        this.getInputValueAsync = (input, data) => __awaiter(this, void 0, void 0, function* () {
            return this.isRule(input) ? this.executeAsync(input, data) : input;
        });
        this.setOutputValue = (out, value) => {
            this.outputs.set(out, value);
        };
        this.getOutputValue = (inputs) => {
            const outputValue = this.outputs.get(inputs[0]);
            if (!outputValue)
                throw Error(`The "${inputs[0]}" output value is not exist`);
            return outputValue;
        };
        this.rules = new Map();
        this.outputs = new Map();
        this.registerOne({ name: 'Variable', shortName: 'Var' }, this.getOutputValue);
    }
    getRuleParams(jsonRules) {
        const rule = jsonRules[core_types_1.RuleParams.Rule] || jsonRules[core_types_1.ShortRuleParams.Rule];
        const inputs = jsonRules[core_types_1.RuleParams.Input] || jsonRules[core_types_1.ShortRuleParams.Input];
        const output = jsonRules[core_types_1.RuleParams.Output] || jsonRules[core_types_1.ShortRuleParams.Output];
        return { rule, inputs, output };
    }
}
exports.RuleCore = RuleCore;
//# sourceMappingURL=core.js.map