export declare type IRulesCore = {
    registerOne(ruleIdentifier: RuleIdentifier, ruleHandler: RuleHandler): void;
    registerMany(rules: Rules): void;
    execute(rules: IJsonRules, data?: {}): RuleResult;
    executeAsync(rules: IJsonRules, data?: {}): Promise<RuleResult>;
};
export declare type RuleIdentifier = {
    name: string;
    shortName?: string;
};
export declare type RuleHandler = ((inputs: RuleInput[], data?: {}) => RuleResult) | ((inputs: RuleInput[], data?: {}) => Promise<RuleResult>);
export declare type RuleInput = string | number | boolean | bigint | string[] | number[] | boolean[] | bigint[] | IJsonRules;
export declare type RuleResult = string | number | boolean | bigint | string[] | number[] | boolean[] | bigint[] | IJsonRules;
export declare type RulesValidation = {
    isValid: boolean;
    errors: string[];
};
export declare enum RuleParams {
    Rule = "Rule",
    Input = "Input",
    Output = "Output"
}
export declare enum ShortRuleParams {
    Rule = "R",
    Input = "I",
    Output = "O"
}
export declare type IJsonRules = {
    [RuleParams.Rule]: string;
    [RuleParams.Input]: string[] | number[] | boolean[] | any[] | IJsonRules[];
    [RuleParams.Output]?: symbol;
} | {
    [ShortRuleParams.Rule]: string;
    [ShortRuleParams.Input]: string[] | number[] | boolean[] | any[] | IJsonRules[];
    [ShortRuleParams.Output]?: symbol;
};
export interface RulesImplementation {
    getRules(): Rules;
}
export declare type Rules = Map<RuleIdentifier, RuleHandler>;
//# sourceMappingURL=core.types.d.ts.map