"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicalRules = void 0;
class LogicalRules {
    constructor() {
        this.register = () => {
            this.rules.set({ name: 'And', shortName: '&&' }, this.and);
            this.rules.set({ name: 'Or', shortName: '||' }, this.or);
            this.rules.set({ name: 'All' }, this.all);
            this.rules.set({ name: 'Equal', shortName: '==' }, this.equal);
            this.rules.set({ name: 'NotEqual', shortName: '!=' }, this.notEqual);
            this.rules.set({ name: 'Not', shortName: '!' }, this.not);
            this.rules.set({ name: 'GreaterThan', shortName: '>' }, this.greaterThan);
            this.rules.set({ name: 'LessThan', shortName: '<' }, this.lessThan);
            this.rules.set({ name: 'GreaterThanOrEqual', shortName: '>=' }, this.greaterThanOrEqual);
            this.rules.set({ name: 'LessThanOrEqual', shortName: '<=' }, this.lessThanOrEqual);
        };
        this.isSameType = (inputs) => {
            return inputs.slice(1).every(i => typeof i === typeof inputs[0]);
        };
        this.and = (inputs) => {
            return inputs.every(input => input);
        };
        this.or = (inputs) => {
            return inputs.some(input => input);
        };
        this.all = (inputs) => {
            return inputs.map(input => input);
        };
        this.equal = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] === inputs[1] : inputs[0] == inputs[1];
        };
        this.notEqual = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] !== inputs[1] : inputs[0] != inputs[1];
        };
        this.not = (inputs) => {
            return !inputs[0];
        };
        this.greaterThan = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] > inputs[1] : false;
        };
        this.lessThan = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] < inputs[1] : false;
        };
        this.greaterThanOrEqual = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] >= inputs[1] : false;
        };
        this.lessThanOrEqual = (inputs) => {
            return this.isSameType(inputs) ? inputs[0] <= inputs[1] : false;
        };
        this.rules = new Map();
        this.register();
    }
    getRules() {
        return this.rules;
    }
}
exports.LogicalRules = LogicalRules;
//# sourceMappingURL=logical.js.map