"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectRules = void 0;
class ObjectRules {
    constructor() {
        this.register = () => {
            this.rules.set({ name: 'Get' }, this.get);
        };
        this.isArrayOfPaths = (path) => {
            if (!Array.isArray(path))
                return false;
            return path.every(p => typeof p === 'string');
        };
        this.baseGet = (pathProps, data) => {
            const property = pathProps.shift();
            if (!property || data[property] === undefined)
                return undefined;
            return pathProps.length === 0 ? (property === '*' ? data : data[property]) : this.baseGet(pathProps, data[property]);
        };
        this.baseSet = (pathProps, data, value, upsert = false) => {
            const property = pathProps.shift();
            if (!property || data[property] === undefined)
                return false;
            if (pathProps.length === 0) {
                data[property] = value;
                return true;
            }
            else {
                return this.baseSet(pathProps, data[property], value);
            }
        };
        this.set = (inputs, data) => {
            const [path, value] = [inputs[0], inputs.length > 1 ? inputs[1] : null];
            const pathProps = this.getPath(path);
            return data && pathProps && value ? this.baseSet(pathProps, data, value, true) : false;
        };
        this.get = (inputs, data) => {
            const [path, defaultValue] = [inputs[0], inputs.length > 1 ? inputs[1] : null];
            const pathProps = this.getPath(path);
            const result = data && pathProps ? this.baseGet(pathProps, data) : undefined;
            return result === undefined ? defaultValue : result;
        };
        this.update = (inputs, data) => {
            const [path, value] = [inputs[0], inputs.length > 1 ? inputs[1] : null];
            const pathProps = this.getPath(path);
            return data && pathProps && value ? this.baseSet(pathProps, data, value) : false;
        };
        this.delete = (inputs, data) => {
        };
        this.arrayEleNumRegex = /^\[([0-9]+)\]/;
        this.rules = new Map();
        this.register();
    }
    getRules() {
        return this.rules;
    }
    getPath(path) {
        let pathProps = path;
        if (typeof path === 'string')
            pathProps = path.split('.');
        return this.isArrayOfPaths(pathProps) && pathProps.length > 0 ? pathProps : undefined;
    }
}
exports.ObjectRules = ObjectRules;
//# sourceMappingURL=object.js.map